@@ -23,7 +23,7 @@
 ;;;  OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 ;;;  IN THE SOFTWARE.
 ;;;
-;;;  $Id: wiliki.scm,v 1.112 2004/03/22 11:44:35 shirok Exp $
+;;;  $Id: wiliki.scm,v 1.116 2005/08/18 04:24:13 shirok Exp $
 ;;;
 
 (define-module wiliki
@@ -300,10 +300,12 @@
 (define (wiliki:search-box)
   `((form (@ (method POST) (action ,(cgi-name-of (wiliki)))
              (style "margin:0pt; padding:0pt"))
-          (input (@ (type hidden) (name c) (value s)
+          (input (@ (type hidden) (name c) (value s)))
+          (input (@ (type text) (name key) (size 15)
                     (style "margin:0pt; padding:0pt")))
-          (span ,($$ "Search:")
-                (input (@ (type text) (name key) (size 10)))))))
+          (input (@ (type submit) (name search) (value ,($$ "Search"))
+                    (style "margin:0pt; padding:0pt")))
+          )))
 
 (define (wiliki:menu-links page)
   `((table
@@ -338,8 +340,11 @@
 
 (define (wiliki:default-head-elements page opts)
   `((title ,(ref page 'title))
-    ,@(or (and-let* ((w (wiliki)))
-            `((base (@ (href ,(full-script-path-of w))))))
+    ,@(or (and-let* ((w (wiliki))
+                     (fsp (full-script-path-of w)))
+            `((base (@ (href ,fsp)))
+              (link (@ (rel "alternative") (type "application/rss+xml")
+                       (title "RSS") (href ,(format "~a?c=rss" fsp))))))
           '())
     ,(or (and-let* ((w (wiliki)) (ss (style-sheet-of w)))
            `(link (@ (rel "stylesheet") (href ,ss) (type "text/css"))))
@@ -391,8 +396,9 @@
   (define (reader-macro-wikiname? name)
     (cond ((string-prefix? "$$" name)
            (handle-reader-macro name))
-          ((or (string-index name #[\s])
-               (string-prefix? "$" name))
+          ((or (string-prefix? "$" name)
+               (#/^\s/ name)
+               (#/\s$/ name))
            ;;invalid wiki name
            (list "[[" name "]]"))
           (else #f)))
@@ -401,24 +407,26 @@
       (or (and head
                (and-let* ((inter-prefix (inter-wikiname-prefix head)))
                  (values inter-prefix after)))
-          (values #f #f))))
-  (receive (prefix inner) (inter-wikiname? name)
-    (cond ((reader-macro-wikiname? name))
-          (prefix
-           (let ((scheme
-                  (if (#/^(https?|ftp|mailto):/ prefix) "" "http://")))
-             `((a (@ (href ,(format "~a~a~a" scheme prefix
-                                    (uri-encode-string (cv-out inner)))))
-                  ,name))))
-          ;; NB: the order of checks here is debatable.  Should a virtual
-          ;; page shadow an existing page, or an existing page shadow a
-          ;; virtual one?  Note also the order of this check must match
-          ;; the order in cmd-view.
-          ((or (wiliki-db-exists? name) (virtual-page? name))
-           (list (wiliki:wikiname-anchor name)))
-          (else
-           `(,name
-             (a (@ (href ,(url "p=~a&c=e" (cv-out name)))) "?")))))
+          (values #f name))))
+  (or (reader-macro-wikiname? name)
+      (receive (inter-prefix real-name) (inter-wikiname? name)
+        (cond (inter-prefix
+               (let1 scheme
+                   (if (#/^(https?|ftp|mailto):/ inter-prefix) "" "http://")
+                 `((a (@ (href ,(format "~a~a~a" scheme inter-prefix
+                                        (uri-encode-string
+                                         (cv-out real-name)))))
+                      ,name))))
+              ;; NB: the order of checks here is debatable.  Should a virtual
+              ;; page shadow an existing page, or an existing page shadow a
+              ;; virtual one?  Note also the order of this check must match
+              ;; the order in cmd-view.
+              ((or (wiliki-db-exists? real-name) (virtual-page? real-name))
+               (list (wiliki:wikiname-anchor real-name)))
+              (else
+               `(,real-name
+                 (a (@ (href ,(url "p=~a&c=e" (cv-out real-name)))) "?")))))
+      )
   )
 
 (wiliki:formatter
