@@ -80,12 +80,13 @@
               (cut ssax:xml->sxml <> '()))))))
 
 (define (page . lines)
-  (make (with-module wiliki <page>)
-    :content (string-join lines "\n" 'suffix)))
+  (string-join lines "\n" 'suffix))
 
 (define-macro (tp label expected-sxml page)
   `(tf ,label ,expected-sxml
-       (list "<result>" (format-content ,page) "</result>")))
+       (list "<result>"
+             (map wiliki:sxml->stree (wiliki:format-content ,page))
+             "</result>")))
 
 ;;------------------------------------------------
 (test-section "inline elements")
@@ -96,7 +97,7 @@
     (page "hoge" "" "huge"))
 (tp "paragraph" '(result (p "hoge\nmoge\n") (p "huge\n"))
     (page "hoge" "moge" "" "huge"))
-(tp "paragraph" '(result (p "hoge\n") (p) (p) (p "huge\n") (p) (p))
+(tp "paragraph" '(result (p "hoge\n") (p "huge\n"))
     (page "" "hoge" "" "" "" "huge" "" ""))
 
 (tp "em" '(result (p "foo" (em "bar") "baz\n"))
@@ -107,15 +108,20 @@
     (page "''foo''''bar''baz''"))
 (tp "em" '(result (p "''foo\nbar''\n"))
     (page "''foo" "bar''"))
+(tp "em(empty)" '(result (p "foo\n"))
+    (page "''''foo"))
 
 (tp "strong" '(result (p "foo" (strong "bar") "baz\n"))
     (page "foo'''bar'''baz"))
-(tp "strong" '(result (p (em "foo" (strong "bar") "baz") "'\n"))
+;; nested em and strong isn't supported well
+'(tp "strong" '(result (p (em "foo" (strong "bar") "baz") "'\n"))
     (page "''foo'''bar'''baz'''"))
 (tp "strong" '(result (p "'" (strong "foo") "'\n"))
     (page "''''foo''''"))
 (tp "strong" '(result (p "'" (strong "foo") "'\n"))
     (page "''''foo''''"))
+(tp "strong(empty)" '(result (p "foo\n"))
+    (page "''''''foo"))
 
 (tp "url" '(result (p (a (@ (href "http://foo")) "http://foo")))
     (page "http://foo"))
@@ -290,7 +296,6 @@
                      (ul (ul (ul (li "def\n")))
                          (li "ghi\n")))
                  (li "jkl\n"))
-             (p)
              (ul (li "xyz\n")))
     (page "- abc"
           "---- def"
@@ -327,7 +332,6 @@
     '(result (ul (li "A\n"
                      (ol (li "a\n")
                          (li "b\n"))))
-             (p)
              (ol (li "B\n"
                      (ul (li "c\n")
                          (li "d\n")))))
@@ -356,7 +360,6 @@
              (ul (li "A\n"
                      (ol (li "a\naa\n"))
                      (ul (li "b\nbb\n"))))
-             (p)
              (ol (ol (ol (li "c\ncc\n")))
                  (ul (li "d\ndd\n"))))
     (page " aaa"
@@ -376,7 +379,6 @@
                      "AA\n")
                  (li "B\n"
                      (pre "cddc\ndccd\n")))
-             (p)
              (pre "eee\n"))
     (page "- A"
           "{{{"
@@ -434,6 +436,21 @@
           "# fff"
           ":ggg:"))
 
+(tp "dl & other list"
+    '(result (ul (li "a\n"
+                     (dl (dt "b\n") (dd (p))))
+                 (li "c\n")))
+    (page "- a"
+          ":b:"
+          "- c"))
+(tp "dl & other list"
+    '(result (ul (ul (li "a\n"
+                         (dl (dt "b\n") (dd (p))))
+                     (li "c\n"))))
+    (page "-- a"
+          ":b:"
+          "-- c"))
+
 ;; NB: the last "iii" should be enclosed by <p>.
 (tp "dl & other blocks"
     '(result (pre " aaa\n")
@@ -473,7 +490,7 @@
     '(result (blockquote (p "aaa\n")))
     (page "<<<" "aaa"))
 (tp "stray closing blockquote"
-    '(result (p "aaa\n>>>"))
+    '(result (p "aaa\n>>>\n"))
     (page "aaa" ">>>"))
 
 (tp "paragraph in blockquote"
@@ -483,8 +500,7 @@
 (tp "nested blockquote"
     '(result (p "aaa\n")
              (blockquote (p "bbb\n")
-                         (blockquote (p)
-                                     (blockquote (p "ccc\n"))
+                         (blockquote (blockquote (p "ccc\n"))
                                      (p "ddd\n"))
                          (p "eee\n"))
              (p "fff\n"))
@@ -502,26 +518,25 @@
           "fff"))
 
 (tp "blockquote & lists"
-    '(result (ul (li (blockquote (p "aaa\n"))
+    '(result (ul (li (blockquote (ul (li "aaa\n")))
                      "bbb\n")
                  (li (blockquote (p "ccc\n")
                                  (ol (li "ddd\n")))))
              (ol (li "eee\n")))
     (page "- "
           "<<<"
-          "aaa"
+          "- aaa"
           ">>>"
           "bbb"
           "- "
           "<<<"
           "ccc"
-          "## ddd"
+          "# ddd"
           ">>>"
           "# eee"))
 
 (tp "blockquote & dl"
-    '(result (blockquote (p)
-                         (dl (dt "aaa\n")
+    '(result (blockquote (dl (dt "aaa\n")
                              (dd (p)
                                  (blockquote (p "bbb\n")
                                              (p "ccc\n"))
@@ -554,7 +569,6 @@
                         (td (@ (class "inbody")) "d\n")
                         (td (@ (class "inbody")) "e\n")
                         (td (@ (class "inbody")) "f\n")))
-             (p)
              (table (@ (class "inbody") (cellspacing "0") (border "1"))
                     (tr (@ (class "inbody"))
                         (td (@ (class "inbody")) "g\n")
@@ -573,7 +587,6 @@
 
 (tp "table & other block elements"
     '(result (blockquote
-              (p)
               (table (@ (class "inbody") (cellspacing "0") (border "1"))
                      (tr (@ (class "inbody"))
                          (td (@ (class "inbody")) "a\n")))
@@ -603,20 +616,54 @@
 ;;------------------------------------------------
 (test-section "block elements (headings)")
 
-(tp "headings" '(result (h2 (a (@ (name "0")) "aa\n"))
-                        (h3 (a (@ (name "1")) "bb\n"))
-                        (h4 (a (@ (name "2")) "cc\n"))
-                        (h5 (a (@ (name "3")) "dd\n"))
-                        (h6 (a (@ (name "4")) "ee\n")))
+(define (hid . lis)
+  (wiliki:calculate-heading-id lis))
+
+(tp "headings"
+    `(result (h2 (@ (id ,(hid "aa"))) "aa\n")
+             (h3 (@ (id ,(hid "bb" "aa"))) "bb\n")
+             (h4 (@ (id ,(hid "cc" "bb" "aa"))) "cc\n")
+             (h5 (@ (id ,(hid "dd" "cc" "bb" "aa"))) "dd\n")
+             (h6 (@ (id ,(hid "ee" "dd" "cc" "bb" "aa"))) "ee\n"))
     (page "* aa" "** bb" "*** cc" "**** dd" "***** ee"))
+(tp "headings (id)"
+    `(result (h2 (@ (id ,(hid "aa"))) "aa\n")
+             (h5 (@ (id ,(hid "bb" "aa"))) "bb\n")
+             (h3 (@ (id ,(hid "cc" "aa"))) "cc\n")
+             (h5 (@ (id ,(hid "bb" "cc" "aa"))) "bb\n")
+             (h2 (@ (id ,(hid "bb"))) "bb\n")
+             (h3 (@ (id ,(hid "cc" "bb"))) "cc\n"))
+    (page "* aa" "**** bb" "** cc" "**** bb" "* bb" "** cc"))
 (tp "headings&list, pre"
-    '(result (p "aaa\n")
-             (h2 (a (@ (name "0")) "aa\n"))
+    `(result (p "aaa\n")
+             (h2 (@ (id ,(hid "aa"))) "aa\n")
              (ul (li "bb\n"))
-             (h3 (a (@ (name "1")) "cc\n"))
+             (h3 (@ (id ,(hid "cc" "aa"))) "cc\n")
              (pre " dd\n")
-             (h4 (a (@ (name "2")) "ee\n")))
+             (h4 (@ (id ,(hid "ee" "cc" "aa"))) "ee\n"))
     (page "aaa" "* aa" "- bb" "** cc" " dd" "*** ee"))
 
+;;------------------------------------------------
+(test-section "wikiname")
+
+;; default handler
+(tp "wikiname"
+    `(result (p "[[aa]]\n"))
+    (page "[[aa]]"))
+(tp "wikiname"
+    `(result (p "bb[[aa]]\n"))
+    (page "bb[[aa]]"))
+(tp "wikiname"
+    `(result (p "bb[[aa\n"))
+    (page "bb[[aa"))
+(tp "wikiname"
+    `(result (p "bb[[aa[[cc\n"))
+    (page "bb[[aa[[cc"))
+(tp "wikiname"
+    `(result (p "bb[[[[cc\n"))
+    (page "bb[[[[cc"))
+(tp "wikiname"
+    `(result (p "bb[[[[cc]]\n"))
+    (page "bb[[[[cc]]"))
 
 (test-end)
